import 'dart:async';

import 'package:xmpp_stone/xmpp_stone.dart';
import 'package:xmpp_stone/src/elements/XmppAttribute.dart';
import 'package:xmpp_stone/src/elements/stanzas/AbstractStanza.dart';

import '../../../../connectycube_core.dart';

import '../../chat_exceptions.dart';
import '../../models/cube_error_packet.dart';
import '../../realtime/managers/base_managers.dart';

class AsyncStanzaSender extends Manager {
  static const int DEFAULT_SOCKET_TIMEOUT = 60;
  static Map<Connection, AsyncStanzaSender> _instances = Map();
  static Map<String, Timer> _timers = Map();
  static Map<String, Function> _tasks = Map();
  StreamSubscription subscription;

  AsyncStanzaSender._private(Connection connection) : super(connection);

  static getInstance(Connection connection) {
    AsyncStanzaSender manager = _instances[connection];
    if (manager == null) {
      manager = AsyncStanzaSender._private(connection);
      _instances[connection] = manager;
    }

    return manager;
  }

  void sendAsync<S extends AbstractStanza>(S stanza, Function func) {
    String id = stanza.id;

    log("add new task $id", "AsyncStanzaSender");
    _tasks[id] = func;
    _timers[id] = _startTimeout(DEFAULT_SOCKET_TIMEOUT, id);

    if (subscription == null) {
      subscription = connection.inStanzasStream.listen((responseStanza) {
        String id = responseStanza.id;
        if (!_tasks.containsKey(id)) return;
        log("run task $id", "AsyncStanzaSender");
        _timers[id].cancel();
        _timers.remove(id);

        Function function = _tasks[id];
        if (function == null) return;
        _tasks.remove(id);

        XmppAttribute type = responseStanza.getAttribute("type");
        if (type != null) {
          if ('error' == type.value) {
            function(
                null,
                ChatConnectionException(
                    error: ErrorPacket.fromStanza(
                        responseStanza.getChild('error'))));
          } else {
            function(responseStanza, null);
          }
        } else {
          function(responseStanza, null);
        }

        if (_tasks.isEmpty) {
          log("pause suncsription", "AsyncStanzaSender");
          subscription.pause();
        }
      });
    } else {
      if (subscription.isPaused) {
        log("unpausepause suncsription", "AsyncStanzaSender");
        subscription.resume();
      }
    }

    connection.writeStanza(stanza);
  }

  _startTimeout(int seconds, String timerId) {
    log("start timer $timerId", "AsyncStanzaSender");
    return new Timer(Duration(seconds: seconds), () => _handleTimeout(timerId));
  }

  void _handleTimeout(String timerId) {
    log("remove task by timer $timerId", "AsyncStanzaSender");
    _timers.remove(timerId);
    Function function = _tasks[timerId];
    if (function == null) return;
    _tasks.remove(timerId);

    function(
        null,
        ChatConnectionException(
            message: 'Connection timeout was occured durins end packet'));

    if (_tasks.isEmpty) {
      log("pause subscription by timeout", "AsyncStanzaSender");
      subscription.pause();
    }
  }
}
