import 'package:device_id/device_id.dart';
import 'package:package_info/package_info.dart';

import '../utils/consts.dart';
import '../utils/string_utils.dart';

class CubeSettings {
  String versionName = "0.0.1-alpha";
//
//  private Context context;

  String applicationId;
  String authorizationKey;
  String accountKey;

  String authorizationSecret;

  String chatDefaultResource = "";

//  private SubscribePushStrategy subscribePushStrategy = SubscribePushStrategy.ALWAYS;
//  private boolean enablePushNotification = true;

//  private LogLevel logLevel = LogLevel.DEBUG;

//  private boolean autoCreateSession = true;

//  private Map<ServiceZone, String> apiEndpointsMap = new HashMap<>();
//  private Map<ServiceZone, String> chatEndpointsMap = new HashMap<>();

//  private ServiceZone zone = ServiceZone.AUTOMATIC;

//  private String restApiVersion = ConstsInternal.REST_API_VERSION;

//  private AutoUpdateMode updateMode;
//  private static String initClassName = "com.connectycube.pushnotifications.ConnectycubePushNotifications";
//  private ConnectycubeSettingsStorage storage;
//  private StoringMechanism storingMechanism = StoringMechanism.SECURED;

  bool isDebugEnabled = true;
  bool isJoinEnabled = true;

  String apiEndpoint = "https://api.connectycube.com";
  String chatEndpoint = "chat.connectycube.com";

  static final CubeSettings _instance = CubeSettings._internal();

  CubeSettings._internal();

  static CubeSettings get instance => _instance;

  init(String applicationId, String authorizationKey,
      String authorizationSecret) async {
    this.applicationId = applicationId;
    this.authorizationKey = authorizationKey;
    this.authorizationSecret = authorizationSecret;

    await _initDefaultParams();
  }

  setEndpoints(String apiEndpoint, String chatEndpoint) {
    if (isEmpty(apiEndpoint) || isEmpty(chatEndpoint)) {
      throw ArgumentError(
          "'apiEndpoint' and(or) 'chatEndpoint' can not be empty or null");
    }

    if (!apiEndpoint.startsWith("http")) {
      apiEndpoint = "https://" + apiEndpoint;
    }

    this.apiEndpoint = apiEndpoint;
    this.chatEndpoint = chatEndpoint;
  }

  Future<void> _initResourceId() async {
    String resourceId = await DeviceId.getID;
    this.chatDefaultResource = "$PREFIX_CHAT_RESOURCE\_$resourceId";
  }

  Future<void> _initVersionName() async {
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    this.versionName = packageInfo.version;
  }

  Future<void> _initDefaultParams() async {
//    await _initVersionName();
    await _initResourceId();
  }
}
